# stdlib
from io import StringIO
import sys
from typing import Any
from typing import List

# third party
import pytest
import torch as th

# syft absolute
import syft as sy
from syft.core.node.common.client import AbstractNodeClient
from syft.core.pointer.pointer import Pointer


def validate_output(data: Any, data_ptr: Pointer) -> None:
    old_stdout = sys.stdout
    sys.stdout = newstdout = StringIO()

    data_ptr.print()

    sys.stdout = old_stdout
    assert newstdout.getvalue().strip("\n") == str(repr(data))


def validate_permission_error(data_ptr: Pointer) -> None:
    old_stdout = sys.stdout
    sys.stdout = newstdout = StringIO()

    data_ptr.print()

    sys.stdout = old_stdout

    assert newstdout.getvalue().startswith("No permission to print")


@pytest.mark.slow
@pytest.mark.parametrize("with_verify_key", [True, False])
def test_make_pointable(
    with_verify_key: bool,
    node: sy.VirtualMachine,
    client: sy.VirtualMachineClient,
    root_client: sy.VirtualMachineClient,
) -> None:
    ten = th.tensor([1, 2])
    ptr = ten.send(root_client, pointable=False)

    assert len(client.store) == 0

    if with_verify_key:
        ptr.update_searchability(target_verify_key=client.verify_key)
    else:
        ptr.update_searchability()

    assert len(client.store) == 1


@pytest.mark.slow
@pytest.mark.parametrize("with_verify_key", [True, False])
def test_make_unpointable(
    with_verify_key: bool,
    node: sy.VirtualMachine,
    client: sy.VirtualMachineClient,
    root_client: sy.VirtualMachineClient,
) -> None:
    ten = th.tensor([1, 2])
    ptr = ten.send(root_client, pointable=False)

    if with_verify_key:
        ptr.update_searchability(target_verify_key=client.verify_key)
    else:
        ptr.update_searchability()

    assert len(client.store) == 1

    if with_verify_key:
        ptr.update_searchability(pointable=False, target_verify_key=client.verify_key)
    else:
        ptr.update_searchability(pointable=False)

    assert len(client.store) == 0


@pytest.mark.slow
def test_pointable_property(
    client: sy.VirtualMachineClient, root_client: sy.VirtualMachineClient
) -> None:
    ten = th.tensor([1, 2])
    ptr = ten.send(root_client, pointable=False)
    assert len(client.store) == 0

    ptr.pointable = False
    assert len(client.store) == 0

    ptr.pointable = True
    assert len(client.store) == 1

    ptr.pointable = True
    assert len(client.store) == 1

    ptr.pointable = False
    assert len(client.store) == 0


@pytest.mark.slow
@pytest.mark.xfail
def test_tags(root_client: sy.VirtualMachineClient) -> None:
    ten = th.tensor([1, 2])

    ten = ten.tag("tag1", "tag1", "other")
    assert ten.tags == ["tag1", "other"]

    # .send without `tags` passed in
    ptr = ten.send(root_client)
    assert ptr.tags == ["tag1", "other"]

    # .send with `tags` passed in
    ptr = ten.send(root_client, tags=["tag2", "tag2", "other"])
    assert ten.tags == ["tag2", "other"]
    assert ptr.tags == ["tag2", "other"]

    th.Tensor([1, 2, 3]).send(root_client, pointable=True, tags=["a"])
    th.Tensor([1, 2, 3]).send(root_client, pointable=True, tags=["b"])
    th.Tensor([1, 2, 3]).send(root_client, pointable=True, tags=["c"])
    th.Tensor([1, 2, 3]).send(root_client, pointable=True, tags=["d"])
    sy.lib.python.Int(2).send(root_client, pointable=True, tags=["e"])
    sy.lib.python.List([1, 2, 3]).send(root_client, pointable=True, tags=["f"])

    a = root_client.store["a"]
    b = root_client.store["b"]
    c = root_client.store["c"]
    d = root_client.store["d"]
    e = root_client.store["e"]

    result_ptr = a.requires_grad
    assert result_ptr.tags == ["a", "requires_grad"]

    result_ptr = b.pow(e)
    assert result_ptr.tags == ["b", "e", "pow"]

    result_ptr = c.pow(exponent=e)
    assert result_ptr.tags == ["c", "e", "pow"]

    result_ptr = root_client.torch.pow(d, e)
    assert result_ptr.tags == ["d", "e", "pow"]

    result_ptr = root_client.torch.pow(d, 3)
    assert result_ptr.tags == ["d", "pow"]

    # __len__ auto gets if you have permission
    f_root = root_client.store["f"]
    assert len(f_root) == 3


def test_issue_5170(client: sy.VirtualMachineClient) -> None:
    sy.lib.python.List([1, 2, 3]).send(client, pointable=True, tags=["f"])

    f_guest = client.store["f"]
    result_ptr = f_guest.len()
    assert result_ptr is not None
    assert result_ptr.tags == ["f", "__len__"]

    with pytest.raises(ValueError) as e:
        f_guest.__len__()

    assert str(e.value) == "Request to access data length rejected."


def test_description(root_client: sy.VirtualMachineClient) -> None:
    ten = th.tensor([1, 2])

    ten = ten.describe("description 1")
    assert ten.description == "description 1"

    # .send without `description` passed in
    ptr = ten.send(root_client)
    assert ptr.description == "description 1"

    # .send with `description` passed in
    ptr = ten.send(root_client, description="description 2")
    assert ten.description == "description 2"
    assert ptr.description == "description 2"


def test_printing(
    client: sy.VirtualMachineClient, root_client: sy.VirtualMachineClient
) -> None:
    data_types = [
        sy.lib.python.Int(1),
        sy.lib.python.Float(1.5),
        sy.lib.python.Bool(True),
        sy.lib.python.List([1, 2, 3]),
        sy.lib.python.Tuple((1, 2, 3)),
        th.tensor([1, 2, 3]),
    ]

    for data in data_types:
        validate_output(data, data.send(root_client))

    for data in data_types:
        validate_permission_error(data.send(client))


def test_printing_remote_creation(
    client: sy.VirtualMachineClient, root_client: sy.VirtualMachineClient
) -> None:
    def create_data_types(client: AbstractNodeClient) -> List[Pointer]:
        return [
            client.syft.lib.python.Int(1),
            client.syft.lib.python.Float(1.5),
            client.syft.lib.python.Bool(True),
            client.syft.lib.python.List([1, 2, 3]),
            client.syft.lib.python.Tuple((1, 2, 3)),
            client.torch.Tensor([1, 2, 3]),
        ]

    for elem in create_data_types(root_client):
        out = elem.get(delete_obj=False)
        validate_output(out, elem)

    for idx, elem in enumerate(create_data_types(client)):
        validate_permission_error(elem)


def test_exhausted(root_client: sy.VirtualMachineClient) -> None:
    int_ptr = root_client.syft.lib.python.Int(0)
    int_ptr.get()  # ptr gets exhausted after this call

    with pytest.raises(ReferenceError) as e:
        int_ptr.get()

    assert str(e.value) == "Object has already been deleted. This pointer is exhausted"



# third party
import numpy as np
import torch

np.random.seed(2)

T = 20
L = 1000
N = 100

x = np.empty((N, L), "int64")
x[:] = np.array(range(L)) + np.random.randint(-4 * T, 4 * T, N).reshape(N, 1)
data = np.sin(x / 1.0 / T).astype("float64")
torch.save(data, open("traindata.pt", "wb"))



# third party
import torch as th

# syft relative
from ...proto.lib.torch.tensor_pb2 import TensorData

# Torch dtypes to string (and back) mappers
TORCH_DTYPE_STR = {
    th.uint8: "uint8",
    th.int8: "int8",
    th.int16: "int16",
    th.int32: "int32",
    th.int64: "int64",
    th.float16: "float16",
    th.float32: "float32",
    th.float64: "float64",
    th.complex32: "complex32",
    th.complex64: "complex64",
    th.complex128: "complex128",
    th.bool: "bool",
    th.qint8: "qint8",
    th.quint8: "quint8",
    th.qint32: "qint32",
    th.bfloat16: "bfloat16",
}
TORCH_STR_DTYPE = {name: cls for cls, name in TORCH_DTYPE_STR.items()}


def protobuf_tensor_serializer(tensor: th.Tensor) -> TensorData:
    """Strategy to serialize a tensor using Protobuf"""
    dtype = TORCH_DTYPE_STR[tensor.dtype]

    protobuf_tensor = TensorData()

    if tensor.is_quantized:
        protobuf_tensor.is_quantized = True
        protobuf_tensor.scale = tensor.q_scale()
        protobuf_tensor.zero_point = tensor.q_zero_point()
        data = th.flatten(tensor).int_repr().tolist()
    else:
        data = th.flatten(tensor).tolist()

    protobuf_tensor.dtype = dtype
    protobuf_tensor.shape.extend(tensor.size())
    getattr(protobuf_tensor, "contents_" + dtype).extend(data)

    return protobuf_tensor


def protobuf_tensor_deserializer(protobuf_tensor: TensorData) -> th.Tensor:
    """Strategy to deserialize a binary input using Protobuf"""
    size = tuple(protobuf_tensor.shape)
    data = getattr(protobuf_tensor, "contents_" + protobuf_tensor.dtype)

    if protobuf_tensor.is_quantized:
        # Drop the 'q' from the beginning of the quantized dtype to get the int type
        dtype = TORCH_STR_DTYPE[protobuf_tensor.dtype[1:]]
        int_tensor = th.tensor(data, dtype=dtype).reshape(size)
        # Automatically converts int types to quantized types
        return th._make_per_tensor_quantized_tensor(
            int_tensor, protobuf_tensor.scale, protobuf_tensor.zero_point
        )
    else:
        dtype = TORCH_STR_DTYPE[protobuf_tensor.dtype]
        return th.tensor(data, dtype=dtype).reshape(size)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/lib/torch/tensor.proto
"""Generated protocol buffer code."""
# third party
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


# syft absolute
from syft.proto.lib.torch import device_pb2 as proto_dot_lib_dot_torch_dot_device__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name="proto/lib/torch/tensor.proto",
    package="syft.lib.torch",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x1cproto/lib/torch/tensor.proto\x12\x0esyft.lib.torch\x1a\x1cproto/lib/torch/device.proto"\xa4\x03\n\nTensorData\x12\r\n\x05shape\x18\x01 \x03(\x03\x12\r\n\x05\x64type\x18\x02 \x01(\t\x12\x14\n\x0cis_quantized\x18\x03 \x01(\x08\x12\r\n\x05scale\x18\x04 \x01(\x02\x12\x12\n\nzero_point\x18\x05 \x01(\x05\x12\x16\n\x0e\x63ontents_uint8\x18\x10 \x03(\r\x12\x15\n\rcontents_int8\x18\x11 \x03(\x05\x12\x16\n\x0e\x63ontents_int16\x18\x12 \x03(\x05\x12\x16\n\x0e\x63ontents_int32\x18\x13 \x03(\x05\x12\x16\n\x0e\x63ontents_int64\x18\x14 \x03(\x03\x12\x18\n\x10\x63ontents_float16\x18\x15 \x03(\x02\x12\x18\n\x10\x63ontents_float32\x18\x16 \x03(\x02\x12\x18\n\x10\x63ontents_float64\x18\x17 \x03(\x01\x12\x15\n\rcontents_bool\x18\x18 \x03(\x08\x12\x16\n\x0e\x63ontents_qint8\x18\x19 \x03(\x11\x12\x17\n\x0f\x63ontents_quint8\x18\x1a \x03(\r\x12\x17\n\x0f\x63ontents_qint32\x18\x1b \x03(\x11\x12\x19\n\x11\x63ontents_bfloat16\x18\x1c \x03(\x02"\xa2\x01\n\x0bTensorProto\x12*\n\x06tensor\x18\x01 \x01(\x0b\x32\x1a.syft.lib.torch.TensorData\x12\x15\n\rrequires_grad\x18\x02 \x01(\x08\x12(\n\x04grad\x18\x03 \x01(\x0b\x32\x1a.syft.lib.torch.TensorData\x12&\n\x06\x64\x65vice\x18\x04 \x01(\x0b\x32\x16.syft.lib.torch.Deviceb\x06proto3',
    dependencies=[
        proto_dot_lib_dot_torch_dot_device__pb2.DESCRIPTOR,
    ],
)


_TENSORDATA = _descriptor.Descriptor(
    name="TensorData",
    full_name="syft.lib.torch.TensorData",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="shape",
            full_name="syft.lib.torch.TensorData.shape",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="dtype",
            full_name="syft.lib.torch.TensorData.dtype",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="is_quantized",
            full_name="syft.lib.torch.TensorData.is_quantized",
            index=2,
            number=3,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="scale",
            full_name="syft.lib.torch.TensorData.scale",
            index=3,
            number=4,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="zero_point",
            full_name="syft.lib.torch.TensorData.zero_point",
            index=4,
            number=5,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="contents_uint8",
            full_name="syft.lib.torch.TensorData.contents_uint8",
            index=5,
            number=16,
            type=13,
            cpp_type=3,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="contents_int8",
            full_name="syft.lib.torch.TensorData.contents_int8",
            index=6,
            number=17,
            type=5,
            cpp_type=1,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="contents_int16",
            full_name="syft.lib.torch.TensorData.contents_int16",
            index=7,
            number=18,
            type=5,
            cpp_type=1,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="contents_int32",
            full_name="syft.lib.torch.TensorData.contents_int32",
            index=8,
            number=19,
            type=5,
            cpp_type=1,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="contents_int64",
            full_name="syft.lib.torch.TensorData.contents_int64",
            index=9,
            number=20,
            type=3,
            cpp_type=2,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="contents_float16",
            full_name="syft.lib.torch.TensorData.contents_float16",
            index=10,
            number=21,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="contents_float32",
            full_name="syft.lib.torch.TensorData.contents_float32",
            index=11,
            number=22,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="contents_float64",
            full_name="syft.lib.torch.TensorData.contents_float64",
            index=12,
            number=23,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="contents_bool",
            full_name="syft.lib.torch.TensorData.contents_bool",
            index=13,
            number=24,
            type=8,
            cpp_type=7,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="contents_qint8",
            full_name="syft.lib.torch.TensorData.contents_qint8",
            index=14,
            number=25,
            type=17,
            cpp_type=1,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="contents_quint8",
            full_name="syft.lib.torch.TensorData.contents_quint8",
            index=15,
            number=26,
            type=13,
            cpp_type=3,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="contents_qint32",
            full_name="syft.lib.torch.TensorData.contents_qint32",
            index=16,
            number=27,
            type=17,
            cpp_type=1,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="contents_bfloat16",
            full_name="syft.lib.torch.TensorData.contents_bfloat16",
            index=17,
            number=28,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=79,
    serialized_end=499,
)


_TENSORPROTO = _descriptor.Descriptor(
    name="TensorProto",
    full_name="syft.lib.torch.TensorProto",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="tensor",
            full_name="syft.lib.torch.TensorProto.tensor",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="requires_grad",
            full_name="syft.lib.torch.TensorProto.requires_grad",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="grad",
            full_name="syft.lib.torch.TensorProto.grad",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="device",
            full_name="syft.lib.torch.TensorProto.device",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=502,
    serialized_end=664,
)

_TENSORPROTO.fields_by_name["tensor"].message_type = _TENSORDATA
_TENSORPROTO.fields_by_name["grad"].message_type = _TENSORDATA
_TENSORPROTO.fields_by_name[
    "device"
].message_type = proto_dot_lib_dot_torch_dot_device__pb2._DEVICE
DESCRIPTOR.message_types_by_name["TensorData"] = _TENSORDATA
DESCRIPTOR.message_types_by_name["TensorProto"] = _TENSORPROTO
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TensorData = _reflection.GeneratedProtocolMessageType(
    "TensorData",
    (_message.Message,),
    {
        "DESCRIPTOR": _TENSORDATA,
        "__module__": "proto.lib.torch.tensor_pb2"
        # @@protoc_insertion_point(class_scope:syft.lib.torch.TensorData)
    },
)
_sym_db.RegisterMessage(TensorData)

TensorProto = _reflection.GeneratedProtocolMessageType(
    "TensorProto",
    (_message.Message,),
    {
        "DESCRIPTOR": _TENSORPROTO,
        "__module__": "proto.lib.torch.tensor_pb2"
        # @@protoc_insertion_point(class_scope:syft.lib.torch.TensorProto)
    },
)
_sym_db.RegisterMessage(TensorProto)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/core/node/domain/service/request_message.proto
"""Generated protocol buffer code."""
# third party
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


# syft absolute
from syft.proto.core.common import (
    common_object_pb2 as proto_dot_core_dot_common_dot_common__object__pb2,
)
from syft.proto.core.io import address_pb2 as proto_dot_core_dot_io_dot_address__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name="proto/core/node/domain/service/request_message.proto",
    package="syft.core.node.domain.service",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n4proto/core/node/domain/service/request_message.proto\x12\x1dsyft.core.node.domain.service\x1a%proto/core/common/common_object.proto\x1a\x1bproto/core/io/address.proto"\xbd\x02\n\x0eRequestMessage\x12)\n\nrequest_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12(\n\tobject_id\x18\x02 \x01(\x0b\x32\x15.syft.core.common.UID\x12\x13\n\x0bobject_tags\x18\x03 \x03(\t\x12\x1b\n\x13request_description\x18\x04 \x01(\t\x12-\n\x0etarget_address\x18\x05 \x01(\x0b\x32\x15.syft.core.io.Address\x12,\n\rowner_address\x18\x06 \x01(\x0b\x32\x15.syft.core.io.Address\x12\x1c\n\x14requester_verify_key\x18\x07 \x01(\x0c\x12\x14\n\x0ctimeout_secs\x18\x08 \x01(\x05\x12\x13\n\x0bobject_type\x18\t \x01(\tb\x06proto3',
    dependencies=[
        proto_dot_core_dot_common_dot_common__object__pb2.DESCRIPTOR,
        proto_dot_core_dot_io_dot_address__pb2.DESCRIPTOR,
    ],
)


_REQUESTMESSAGE = _descriptor.Descriptor(
    name="RequestMessage",
    full_name="syft.core.node.domain.service.RequestMessage",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="request_id",
            full_name="syft.core.node.domain.service.RequestMessage.request_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="object_id",
            full_name="syft.core.node.domain.service.RequestMessage.object_id",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="object_tags",
            full_name="syft.core.node.domain.service.RequestMessage.object_tags",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="request_description",
            full_name="syft.core.node.domain.service.RequestMessage.request_description",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="target_address",
            full_name="syft.core.node.domain.service.RequestMessage.target_address",
            index=4,
            number=5,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="owner_address",
            full_name="syft.core.node.domain.service.RequestMessage.owner_address",
            index=5,
            number=6,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="requester_verify_key",
            full_name="syft.core.node.domain.service.RequestMessage.requester_verify_key",
            index=6,
            number=7,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"",
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="timeout_secs",
            full_name="syft.core.node.domain.service.RequestMessage.timeout_secs",
            index=7,
            number=8,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="object_type",
            full_name="syft.core.node.domain.service.RequestMessage.object_type",
            index=8,
            number=9,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=156,
    serialized_end=473,
)

_REQUESTMESSAGE.fields_by_name[
    "request_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_REQUESTMESSAGE.fields_by_name[
    "object_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_REQUESTMESSAGE.fields_by_name[
    "target_address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_REQUESTMESSAGE.fields_by_name[
    "owner_address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
DESCRIPTOR.message_types_by_name["RequestMessage"] = _REQUESTMESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

RequestMessage = _reflection.GeneratedProtocolMessageType(
    "RequestMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _REQUESTMESSAGE,
        "__module__": "proto.core.node.domain.service.request_message_pb2"
        # @@protoc_insertion_point(class_scope:syft.core.node.domain.service.RequestMessage)
    },
)
_sym_db.RegisterMessage(RequestMessage)


# @@protoc_insertion_point(module_scope)



# third party
from google.protobuf.reflection import GeneratedProtocolMessageType

# syft relative
from ...core.common.message import SignedEventualSyftMessageWithoutReply
from ...core.common.message import SignedImmediateSyftMessageWithReply
from ...core.common.message import SignedImmediateSyftMessageWithoutReply
from ...logger import traceback_and_raise


class BidirectionalConnection(object):
    def recv_immediate_msg_with_reply(
        self, msg: SignedImmediateSyftMessageWithReply
    ) -> SignedImmediateSyftMessageWithoutReply:
        traceback_and_raise(NotImplementedError)

    def recv_immediate_msg_without_reply(
        self, msg: SignedImmediateSyftMessageWithoutReply
    ) -> None:
        traceback_and_raise(NotImplementedError)

    def recv_eventual_msg_without_reply(
        self, msg: SignedEventualSyftMessageWithoutReply
    ) -> None:
        traceback_and_raise(NotImplementedError)

    def send_immediate_msg_with_reply(
        self, msg: SignedImmediateSyftMessageWithReply
    ) -> SignedImmediateSyftMessageWithoutReply:
        traceback_and_raise(NotImplementedError)

    def send_immediate_msg_without_reply(
        self, msg: SignedImmediateSyftMessageWithoutReply
    ) -> None:
        traceback_and_raise(NotImplementedError)

    def send_eventual_msg_without_reply(
        self, msg: SignedEventualSyftMessageWithoutReply
    ) -> None:
        traceback_and_raise(NotImplementedError)

    def _object2proto(self) -> None:
        traceback_and_raise(NotImplementedError)

    @staticmethod
    def _proto2object(proto: GeneratedProtocolMessageType) -> "BidirectionalConnection":
        traceback_and_raise(NotImplementedError)

    @staticmethod
    def get_protobuf_schema() -> GeneratedProtocolMessageType:
        traceback_and_raise(NotImplementedError)


class ServerConnection(object):
    def __init__(self) -> None:
        self.opt_bidirectional_conn = BidirectionalConnection()

    def recv_immediate_msg_with_reply(
        self, msg: SignedImmediateSyftMessageWithReply
    ) -> SignedImmediateSyftMessageWithoutReply:
        traceback_and_raise(NotImplementedError)

    def recv_immediate_msg_without_reply(
        self, msg: SignedImmediateSyftMessageWithoutReply
    ) -> None:
        traceback_and_raise(NotImplementedError)

    def recv_eventual_msg_without_reply(
        self, msg: SignedEventualSyftMessageWithoutReply
    ) -> None:
        traceback_and_raise(NotImplementedError)

    def _object2proto(self) -> None:
        traceback_and_raise(NotImplementedError)

    @staticmethod
    def _proto2object(proto: GeneratedProtocolMessageType) -> "ServerConnection":
        traceback_and_raise(NotImplementedError)

    @staticmethod
    def get_protobuf_schema() -> GeneratedProtocolMessageType:
        traceback_and_raise(NotImplementedError)


class ClientConnection(object):
    def __init__(self) -> None:
        self.opt_bidirectional_conn = BidirectionalConnection()

    def send_immediate_msg_with_reply(
        self, msg: SignedImmediateSyftMessageWithReply
    ) -> SignedImmediateSyftMessageWithoutReply:
        traceback_and_raise(NotImplementedError)

    def send_immediate_msg_without_reply(
        self, msg: SignedImmediateSyftMessageWithoutReply
    ) -> None:
        traceback_and_raise(NotImplementedError)

    def send_eventual_msg_without_reply(
        self, msg: SignedEventualSyftMessageWithoutReply
    ) -> None:
        traceback_and_raise(NotImplementedError)

    def _object2proto(self) -> None:
        traceback_and_raise(NotImplementedError)

    @staticmethod
    def _proto2object(proto: GeneratedProtocolMessageType) -> object:
        traceback_and_raise(NotImplementedError)

    @staticmethod
    def get_protobuf_schema() -> GeneratedProtocolMessageType:
        traceback_and_raise(NotImplementedError)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/grid/messages/transfer_messages.proto
"""Generated protocol buffer code."""
# third party
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


# syft absolute
from syft.proto.core.common import (
    common_object_pb2 as proto_dot_core_dot_common_dot_common__object__pb2,
)
from syft.proto.core.io import address_pb2 as proto_dot_core_dot_io_dot_address__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name="proto/grid/messages/transfer_messages.proto",
    package="syft.grid.messages",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n+proto/grid/messages/transfer_messages.proto\x12\x12syft.grid.messages\x1a%proto/core/common/common_object.proto\x1a\x1bproto/core/io/address.proto"\x9c\x01\n\x11LoadObjectMessage\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12&\n\x07\x61\x64\x64ress\x18\x02 \x01(\x0b\x32\x15.syft.core.io.Address\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12\'\n\x08reply_to\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x89\x01\n\x12LoadObjectResponse\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12\x13\n\x0bstatus_code\x18\x02 \x01(\x05\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12&\n\x07\x61\x64\x64ress\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"s\n\x11SaveObjectMessage\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12&\n\x07\x61\x64\x64ress\x18\x02 \x01(\x0b\x32\x15.syft.core.io.Address\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t"\x89\x01\n\x12SaveObjectResponse\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12\x13\n\x0bstatus_code\x18\x02 \x01(\x05\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12&\n\x07\x61\x64\x64ress\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Addressb\x06proto3',
    dependencies=[
        proto_dot_core_dot_common_dot_common__object__pb2.DESCRIPTOR,
        proto_dot_core_dot_io_dot_address__pb2.DESCRIPTOR,
    ],
)


_LOADOBJECTMESSAGE = _descriptor.Descriptor(
    name="LoadObjectMessage",
    full_name="syft.grid.messages.LoadObjectMessage",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.LoadObjectMessage.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.LoadObjectMessage.address",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.LoadObjectMessage.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="reply_to",
            full_name="syft.grid.messages.LoadObjectMessage.reply_to",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=136,
    serialized_end=292,
)


_LOADOBJECTRESPONSE = _descriptor.Descriptor(
    name="LoadObjectResponse",
    full_name="syft.grid.messages.LoadObjectResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.LoadObjectResponse.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="status_code",
            full_name="syft.grid.messages.LoadObjectResponse.status_code",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.LoadObjectResponse.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.LoadObjectResponse.address",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=295,
    serialized_end=432,
)


_SAVEOBJECTMESSAGE = _descriptor.Descriptor(
    name="SaveObjectMessage",
    full_name="syft.grid.messages.SaveObjectMessage",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.SaveObjectMessage.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.SaveObjectMessage.address",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.SaveObjectMessage.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=434,
    serialized_end=549,
)


_SAVEOBJECTRESPONSE = _descriptor.Descriptor(
    name="SaveObjectResponse",
    full_name="syft.grid.messages.SaveObjectResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.SaveObjectResponse.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="status_code",
            full_name="syft.grid.messages.SaveObjectResponse.status_code",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.SaveObjectResponse.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.SaveObjectResponse.address",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=552,
    serialized_end=689,
)

_LOADOBJECTMESSAGE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_LOADOBJECTMESSAGE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_LOADOBJECTMESSAGE.fields_by_name[
    "reply_to"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_LOADOBJECTRESPONSE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_LOADOBJECTRESPONSE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_SAVEOBJECTMESSAGE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_SAVEOBJECTMESSAGE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_SAVEOBJECTRESPONSE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_SAVEOBJECTRESPONSE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
DESCRIPTOR.message_types_by_name["LoadObjectMessage"] = _LOADOBJECTMESSAGE
DESCRIPTOR.message_types_by_name["LoadObjectResponse"] = _LOADOBJECTRESPONSE
DESCRIPTOR.message_types_by_name["SaveObjectMessage"] = _SAVEOBJECTMESSAGE
DESCRIPTOR.message_types_by_name["SaveObjectResponse"] = _SAVEOBJECTRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

LoadObjectMessage = _reflection.GeneratedProtocolMessageType(
    "LoadObjectMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _LOADOBJECTMESSAGE,
        "__module__": "proto.grid.messages.transfer_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.LoadObjectMessage)
    },
)
_sym_db.RegisterMessage(LoadObjectMessage)

LoadObjectResponse = _reflection.GeneratedProtocolMessageType(
    "LoadObjectResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _LOADOBJECTRESPONSE,
        "__module__": "proto.grid.messages.transfer_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.LoadObjectResponse)
    },
)
_sym_db.RegisterMessage(LoadObjectResponse)

SaveObjectMessage = _reflection.GeneratedProtocolMessageType(
    "SaveObjectMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _SAVEOBJECTMESSAGE,
        "__module__": "proto.grid.messages.transfer_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.SaveObjectMessage)
    },
)
_sym_db.RegisterMessage(SaveObjectMessage)

SaveObjectResponse = _reflection.GeneratedProtocolMessageType(
    "SaveObjectResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _SAVEOBJECTRESPONSE,
        "__module__": "proto.grid.messages.transfer_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.SaveObjectResponse)
    },
)
_sym_db.RegisterMessage(SaveObjectResponse)


# @@protoc_insertion_point(module_scope)



# stdlib
import socket
import time
from typing import Any
from typing import List

# syft absolute
from syft.lib.python import List as SyList
from syft.lib.python.string import String

# syft relative
from ...syft.grid.duet.process_test import SyftTestProcess

PORT = 21211


def do_send(data: Any) -> None:
    # syft absolute
    import syft as sy

    duet = sy.launch_duet(loopback=True, network_url=f"http://127.0.0.1:{PORT}/")
    duet.requests.add_handler(action="accept")

    _ = data.send(duet, pointable=True)

    sy.core.common.event_loop.loop.run_forever()


def ds_get(data: Any) -> None:
    # syft absolute
    import syft as sy

    duet = sy.join_duet(loopback=True, network_url=f"http://127.0.0.1:{PORT}/")

    for retry in range(10):
        if len(duet.store) != 0:
            break
        time.sleep(0.1)

    assert len(duet.store) != 0

    remote = duet.store[0].get(request_block=True, delete_obj=False)

    assert remote == data


def run_endpoints(do_runner: Any, ds_runner: Any, data: Any) -> None:
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        assert s.connect_ex(("localhost", PORT)) == 0

    do_proc = SyftTestProcess(target=do_runner, args=(data,))
    do_proc.start()

    ds_proc = SyftTestProcess(target=ds_runner, args=(data,))
    ds_proc.start()

    ds_proc.join(120)

    do_proc.terminate()

    if do_proc.exception:
        exception, tb = do_proc.exception
        raise Exception(tb) from exception

    if ds_proc.exception:
        exception, tb = ds_proc.exception
        raise Exception(tb) from exception

    if ds_proc.is_alive():
        ds_proc.terminate()
        raise Exception(f"ds_proc is hanged for {len(data)}")


def send_get_string_multiprocess(data: String) -> None:
    run_endpoints(do_send, ds_get, String(data))


def send_get_list_multiprocess(data: List[str]) -> None:
    run_endpoints(do_send, ds_get, SyList(data))



# stdlib
import sys
import time


def do_test(port: int) -> None:
    # syft absolute
    import syft as sy

    sy.logger.add(sys.stderr, "ERROR")

    duet = sy.launch_duet(loopback=True, network_url=f"http://127.0.0.1:{port}/")
    duet.requests.add_handler(action="accept")

    _ = sy.lib.python.List([1, 2, 3]).send(duet, pointable=True)

    sy.core.common.event_loop.loop.run_forever()


def ds_test(port: int) -> None:
    # syft absolute
    import syft as sy

    sy.logger.add(sys.stderr, "ERROR")

    duet = sy.join_duet(loopback=True, network_url=f"http://127.0.0.1:{port}/")

    time.sleep(1)

    data_ptr = duet.store[0]
    data = data_ptr.get(request_block=True, delete_obj=False)

    assert data == [1, 2, 3]


test_scenario_sanity = ("test_scenario_sanity", do_test, ds_test)



# syft absolute
from syft import deserialize
from syft import serialize
from syft.core.common import UID
from syft.core.io.address import Address
from syft.core.node.domain.service import RequestAnswerMessage
from syft.core.node.domain.service import RequestAnswerResponse
from syft.core.node.domain.service import RequestStatus


def test_request_answer_message() -> None:

    addr = Address()

    msg = RequestAnswerMessage(request_id=UID(), address=addr, reply_to=addr)

    serialized = serialize(obj=msg)
    new_msg = deserialize(blob=serialized)

    assert msg.request_id == new_msg.request_id
    assert msg.address == new_msg.address
    assert msg.reply_to == new_msg.reply_to


def test_request_answer_response() -> None:

    addr = Address()

    msg = RequestAnswerResponse(
        request_id=UID(), address=addr, status=RequestStatus.Pending
    )

    serialized = serialize(obj=msg)
    new_msg = deserialize(blob=serialized)

    assert msg.request_id == new_msg.request_id
    assert msg.address == new_msg.address
    assert msg.status == new_msg.status



# third party
import petlib as pl

# syft relative
from ...generate_wrapper import GenerateWrapper
from ...proto.lib.petlib.bn_pb2 import Bn as Bn_PB


def object2proto_group(obj: pl.bn.Bn) -> Bn_PB:
    proto = Bn_PB()
    print("what do we have???", obj, obj.hex(), type(obj.hex()))
    proto.hex = obj.hex()
    return proto


def proto2object_group(proto: Bn_PB) -> pl.bn.Bn:
    return pl.bn.Bn.from_hex(proto.hex)


GenerateWrapper(
    wrapped_type=pl.bn.Bn,
    import_path="petlib.bn.Bn",
    protobuf_scheme=Bn_PB,
    type_object2proto=object2proto_group,
    type_proto2object=proto2object_group,
)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/core/node/common/client.proto
"""Generated protocol buffer code."""
# third party
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


# syft absolute
from syft.proto.core.common import (
    common_object_pb2 as proto_dot_core_dot_common_dot_common__object__pb2,
)
from syft.proto.core.io import location_pb2 as proto_dot_core_dot_io_dot_location__pb2
from syft.proto.core.io import route_pb2 as proto_dot_core_dot_io_dot_route__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name="proto/core/node/common/client.proto",
    package="syft.core.node.common",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b"\n#proto/core/node/common/client.proto\x12\x15syft.core.node.common\x1a%proto/core/common/common_object.proto\x1a\x1cproto/core/io/location.proto\x1a\x19proto/core/io/route.proto\"\xfe\x02\n\x06\x43lient\x12\x10\n\x08obj_type\x18\x01 \x01(\t\x12!\n\x02id\x18\x02 \x01(\x0b\x32\x15.syft.core.common.UID\x12\x0c\n\x04name\x18\x03 \x01(\t\x12'\n\x06routes\x18\x04 \x03(\x0b\x32\x17.syft.core.io.SoloRoute\x12\x13\n\x0bhas_network\x18\x05 \x01(\x08\x12/\n\x07network\x18\x06 \x01(\x0b\x32\x1e.syft.core.io.SpecificLocation\x12\x12\n\nhas_domain\x18\x07 \x01(\x08\x12.\n\x06\x64omain\x18\x08 \x01(\x0b\x32\x1e.syft.core.io.SpecificLocation\x12\x12\n\nhas_device\x18\t \x01(\x08\x12.\n\x06\x64\x65vice\x18\n \x01(\x0b\x32\x1e.syft.core.io.SpecificLocation\x12\x0e\n\x06has_vm\x18\x0b \x01(\x08\x12*\n\x02vm\x18\x0c \x01(\x0b\x32\x1e.syft.core.io.SpecificLocationb\x06proto3",
    dependencies=[
        proto_dot_core_dot_common_dot_common__object__pb2.DESCRIPTOR,
        proto_dot_core_dot_io_dot_location__pb2.DESCRIPTOR,
        proto_dot_core_dot_io_dot_route__pb2.DESCRIPTOR,
    ],
)


_CLIENT = _descriptor.Descriptor(
    name="Client",
    full_name="syft.core.node.common.Client",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="obj_type",
            full_name="syft.core.node.common.Client.obj_type",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="id",
            full_name="syft.core.node.common.Client.id",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="name",
            full_name="syft.core.node.common.Client.name",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="routes",
            full_name="syft.core.node.common.Client.routes",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="has_network",
            full_name="syft.core.node.common.Client.has_network",
            index=4,
            number=5,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="network",
            full_name="syft.core.node.common.Client.network",
            index=5,
            number=6,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="has_domain",
            full_name="syft.core.node.common.Client.has_domain",
            index=6,
            number=7,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="domain",
            full_name="syft.core.node.common.Client.domain",
            index=7,
            number=8,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="has_device",
            full_name="syft.core.node.common.Client.has_device",
            index=8,
            number=9,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="device",
            full_name="syft.core.node.common.Client.device",
            index=9,
            number=10,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="has_vm",
            full_name="syft.core.node.common.Client.has_vm",
            index=10,
            number=11,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="vm",
            full_name="syft.core.node.common.Client.vm",
            index=11,
            number=12,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=159,
    serialized_end=541,
)

_CLIENT.fields_by_name[
    "id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_CLIENT.fields_by_name[
    "routes"
].message_type = proto_dot_core_dot_io_dot_route__pb2._SOLOROUTE
_CLIENT.fields_by_name[
    "network"
].message_type = proto_dot_core_dot_io_dot_location__pb2._SPECIFICLOCATION
_CLIENT.fields_by_name[
    "domain"
].message_type = proto_dot_core_dot_io_dot_location__pb2._SPECIFICLOCATION
_CLIENT.fields_by_name[
    "device"
].message_type = proto_dot_core_dot_io_dot_location__pb2._SPECIFICLOCATION
_CLIENT.fields_by_name[
    "vm"
].message_type = proto_dot_core_dot_io_dot_location__pb2._SPECIFICLOCATION
DESCRIPTOR.message_types_by_name["Client"] = _CLIENT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Client = _reflection.GeneratedProtocolMessageType(
    "Client",
    (_message.Message,),
    {
        "DESCRIPTOR": _CLIENT,
        "__module__": "proto.core.node.common.client_pb2"
        # @@protoc_insertion_point(class_scope:syft.core.node.common.Client)
    },
)
_sym_db.RegisterMessage(Client)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/grid/messages/tensor_messages.proto
"""Generated protocol buffer code."""
# third party
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


# syft absolute
from syft.proto.core.common import (
    common_object_pb2 as proto_dot_core_dot_common_dot_common__object__pb2,
)
from syft.proto.core.io import address_pb2 as proto_dot_core_dot_io_dot_address__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name="proto/grid/messages/tensor_messages.proto",
    package="syft.grid.messages",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n)proto/grid/messages/tensor_messages.proto\x12\x12syft.grid.messages\x1a%proto/core/common/common_object.proto\x1a\x1bproto/core/io/address.proto"\x9e\x01\n\x13\x43reateTensorMessage\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12&\n\x07\x61\x64\x64ress\x18\x02 \x01(\x0b\x32\x15.syft.core.io.Address\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12\'\n\x08reply_to\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x8b\x01\n\x14\x43reateTensorResponse\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12\x13\n\x0bstatus_code\x18\x02 \x01(\x05\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12&\n\x07\x61\x64\x64ress\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x9b\x01\n\x10GetTensorMessage\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12&\n\x07\x61\x64\x64ress\x18\x02 \x01(\x0b\x32\x15.syft.core.io.Address\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12\'\n\x08reply_to\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x88\x01\n\x11GetTensorResponse\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12\x13\n\x0bstatus_code\x18\x02 \x01(\x05\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12&\n\x07\x61\x64\x64ress\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x9c\x01\n\x11GetTensorsMessage\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12&\n\x07\x61\x64\x64ress\x18\x02 \x01(\x0b\x32\x15.syft.core.io.Address\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12\'\n\x08reply_to\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x89\x01\n\x12GetTensorsResponse\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12\x13\n\x0bstatus_code\x18\x02 \x01(\x05\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12&\n\x07\x61\x64\x64ress\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x9e\x01\n\x13\x44\x65leteTensorMessage\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12&\n\x07\x61\x64\x64ress\x18\x02 \x01(\x0b\x32\x15.syft.core.io.Address\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12\'\n\x08reply_to\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x8b\x01\n\x14\x44\x65leteTensorResponse\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12\x13\n\x0bstatus_code\x18\x02 \x01(\x05\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12&\n\x07\x61\x64\x64ress\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x9e\x01\n\x13UpdateTensorMessage\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12&\n\x07\x61\x64\x64ress\x18\x02 \x01(\x0b\x32\x15.syft.core.io.Address\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12\'\n\x08reply_to\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Address"\x8b\x01\n\x14UpdateTensorResponse\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12\x13\n\x0bstatus_code\x18\x02 \x01(\x05\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12&\n\x07\x61\x64\x64ress\x18\x04 \x01(\x0b\x32\x15.syft.core.io.Addressb\x06proto3',
    dependencies=[
        proto_dot_core_dot_common_dot_common__object__pb2.DESCRIPTOR,
        proto_dot_core_dot_io_dot_address__pb2.DESCRIPTOR,
    ],
)


_CREATETENSORMESSAGE = _descriptor.Descriptor(
    name="CreateTensorMessage",
    full_name="syft.grid.messages.CreateTensorMessage",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.CreateTensorMessage.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.CreateTensorMessage.address",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.CreateTensorMessage.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="reply_to",
            full_name="syft.grid.messages.CreateTensorMessage.reply_to",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=134,
    serialized_end=292,
)


_CREATETENSORRESPONSE = _descriptor.Descriptor(
    name="CreateTensorResponse",
    full_name="syft.grid.messages.CreateTensorResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.CreateTensorResponse.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="status_code",
            full_name="syft.grid.messages.CreateTensorResponse.status_code",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.CreateTensorResponse.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.CreateTensorResponse.address",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=295,
    serialized_end=434,
)


_GETTENSORMESSAGE = _descriptor.Descriptor(
    name="GetTensorMessage",
    full_name="syft.grid.messages.GetTensorMessage",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.GetTensorMessage.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.GetTensorMessage.address",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.GetTensorMessage.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="reply_to",
            full_name="syft.grid.messages.GetTensorMessage.reply_to",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=437,
    serialized_end=592,
)


_GETTENSORRESPONSE = _descriptor.Descriptor(
    name="GetTensorResponse",
    full_name="syft.grid.messages.GetTensorResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.GetTensorResponse.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="status_code",
            full_name="syft.grid.messages.GetTensorResponse.status_code",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.GetTensorResponse.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.GetTensorResponse.address",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=595,
    serialized_end=731,
)


_GETTENSORSMESSAGE = _descriptor.Descriptor(
    name="GetTensorsMessage",
    full_name="syft.grid.messages.GetTensorsMessage",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.GetTensorsMessage.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.GetTensorsMessage.address",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.GetTensorsMessage.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="reply_to",
            full_name="syft.grid.messages.GetTensorsMessage.reply_to",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=734,
    serialized_end=890,
)


_GETTENSORSRESPONSE = _descriptor.Descriptor(
    name="GetTensorsResponse",
    full_name="syft.grid.messages.GetTensorsResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.GetTensorsResponse.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="status_code",
            full_name="syft.grid.messages.GetTensorsResponse.status_code",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.GetTensorsResponse.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.GetTensorsResponse.address",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=893,
    serialized_end=1030,
)


_DELETETENSORMESSAGE = _descriptor.Descriptor(
    name="DeleteTensorMessage",
    full_name="syft.grid.messages.DeleteTensorMessage",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.DeleteTensorMessage.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.DeleteTensorMessage.address",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.DeleteTensorMessage.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="reply_to",
            full_name="syft.grid.messages.DeleteTensorMessage.reply_to",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1033,
    serialized_end=1191,
)


_DELETETENSORRESPONSE = _descriptor.Descriptor(
    name="DeleteTensorResponse",
    full_name="syft.grid.messages.DeleteTensorResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.DeleteTensorResponse.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="status_code",
            full_name="syft.grid.messages.DeleteTensorResponse.status_code",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.DeleteTensorResponse.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.DeleteTensorResponse.address",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1194,
    serialized_end=1333,
)


_UPDATETENSORMESSAGE = _descriptor.Descriptor(
    name="UpdateTensorMessage",
    full_name="syft.grid.messages.UpdateTensorMessage",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.UpdateTensorMessage.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.UpdateTensorMessage.address",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.UpdateTensorMessage.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="reply_to",
            full_name="syft.grid.messages.UpdateTensorMessage.reply_to",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1336,
    serialized_end=1494,
)


_UPDATETENSORRESPONSE = _descriptor.Descriptor(
    name="UpdateTensorResponse",
    full_name="syft.grid.messages.UpdateTensorResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg_id",
            full_name="syft.grid.messages.UpdateTensorResponse.msg_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="status_code",
            full_name="syft.grid.messages.UpdateTensorResponse.status_code",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="syft.grid.messages.UpdateTensorResponse.content",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="address",
            full_name="syft.grid.messages.UpdateTensorResponse.address",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1497,
    serialized_end=1636,
)

_CREATETENSORMESSAGE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_CREATETENSORMESSAGE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_CREATETENSORMESSAGE.fields_by_name[
    "reply_to"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_CREATETENSORRESPONSE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_CREATETENSORRESPONSE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_GETTENSORMESSAGE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_GETTENSORMESSAGE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_GETTENSORMESSAGE.fields_by_name[
    "reply_to"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_GETTENSORRESPONSE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_GETTENSORRESPONSE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_GETTENSORSMESSAGE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_GETTENSORSMESSAGE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_GETTENSORSMESSAGE.fields_by_name[
    "reply_to"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_GETTENSORSRESPONSE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_GETTENSORSRESPONSE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_DELETETENSORMESSAGE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_DELETETENSORMESSAGE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_DELETETENSORMESSAGE.fields_by_name[
    "reply_to"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_DELETETENSORRESPONSE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_DELETETENSORRESPONSE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_UPDATETENSORMESSAGE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_UPDATETENSORMESSAGE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_UPDATETENSORMESSAGE.fields_by_name[
    "reply_to"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
_UPDATETENSORRESPONSE.fields_by_name[
    "msg_id"
].message_type = proto_dot_core_dot_common_dot_common__object__pb2._UID
_UPDATETENSORRESPONSE.fields_by_name[
    "address"
].message_type = proto_dot_core_dot_io_dot_address__pb2._ADDRESS
DESCRIPTOR.message_types_by_name["CreateTensorMessage"] = _CREATETENSORMESSAGE
DESCRIPTOR.message_types_by_name["CreateTensorResponse"] = _CREATETENSORRESPONSE
DESCRIPTOR.message_types_by_name["GetTensorMessage"] = _GETTENSORMESSAGE
DESCRIPTOR.message_types_by_name["GetTensorResponse"] = _GETTENSORRESPONSE
DESCRIPTOR.message_types_by_name["GetTensorsMessage"] = _GETTENSORSMESSAGE
DESCRIPTOR.message_types_by_name["GetTensorsResponse"] = _GETTENSORSRESPONSE
DESCRIPTOR.message_types_by_name["DeleteTensorMessage"] = _DELETETENSORMESSAGE
DESCRIPTOR.message_types_by_name["DeleteTensorResponse"] = _DELETETENSORRESPONSE
DESCRIPTOR.message_types_by_name["UpdateTensorMessage"] = _UPDATETENSORMESSAGE
DESCRIPTOR.message_types_by_name["UpdateTensorResponse"] = _UPDATETENSORRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CreateTensorMessage = _reflection.GeneratedProtocolMessageType(
    "CreateTensorMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _CREATETENSORMESSAGE,
        "__module__": "proto.grid.messages.tensor_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.CreateTensorMessage)
    },
)
_sym_db.RegisterMessage(CreateTensorMessage)

CreateTensorResponse = _reflection.GeneratedProtocolMessageType(
    "CreateTensorResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _CREATETENSORRESPONSE,
        "__module__": "proto.grid.messages.tensor_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.CreateTensorResponse)
    },
)
_sym_db.RegisterMessage(CreateTensorResponse)

GetTensorMessage = _reflection.GeneratedProtocolMessageType(
    "GetTensorMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETTENSORMESSAGE,
        "__module__": "proto.grid.messages.tensor_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.GetTensorMessage)
    },
)
_sym_db.RegisterMessage(GetTensorMessage)

GetTensorResponse = _reflection.GeneratedProtocolMessageType(
    "GetTensorResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETTENSORRESPONSE,
        "__module__": "proto.grid.messages.tensor_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.GetTensorResponse)
    },
)
_sym_db.RegisterMessage(GetTensorResponse)

GetTensorsMessage = _reflection.GeneratedProtocolMessageType(
    "GetTensorsMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETTENSORSMESSAGE,
        "__module__": "proto.grid.messages.tensor_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.GetTensorsMessage)
    },
)
_sym_db.RegisterMessage(GetTensorsMessage)

GetTensorsResponse = _reflection.GeneratedProtocolMessageType(
    "GetTensorsResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETTENSORSRESPONSE,
        "__module__": "proto.grid.messages.tensor_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.GetTensorsResponse)
    },
)
_sym_db.RegisterMessage(GetTensorsResponse)

DeleteTensorMessage = _reflection.GeneratedProtocolMessageType(
    "DeleteTensorMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _DELETETENSORMESSAGE,
        "__module__": "proto.grid.messages.tensor_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.DeleteTensorMessage)
    },
)
_sym_db.RegisterMessage(DeleteTensorMessage)

DeleteTensorResponse = _reflection.GeneratedProtocolMessageType(
    "DeleteTensorResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _DELETETENSORRESPONSE,
        "__module__": "proto.grid.messages.tensor_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.DeleteTensorResponse)
    },
)
_sym_db.RegisterMessage(DeleteTensorResponse)

UpdateTensorMessage = _reflection.GeneratedProtocolMessageType(
    "UpdateTensorMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _UPDATETENSORMESSAGE,
        "__module__": "proto.grid.messages.tensor_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.UpdateTensorMessage)
    },
)
_sym_db.RegisterMessage(UpdateTensorMessage)

UpdateTensorResponse = _reflection.GeneratedProtocolMessageType(
    "UpdateTensorResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _UPDATETENSORRESPONSE,
        "__module__": "proto.grid.messages.tensor_messages_pb2"
        # @@protoc_insertion_point(class_scope:syft.grid.messages.UpdateTensorResponse)
    },
)
_sym_db.RegisterMessage(UpdateTensorResponse)


# @@protoc_insertion_point(module_scope)



# stdlib
from typing import Any
from typing import Optional

# third party
from google.protobuf.reflection import GeneratedProtocolMessageType

# syft relative
from ... import deserialize
from ... import serialize
from ...core.common import UID
from ...core.common.serde.serializable import bind_protobuf
from ...proto.lib.python.none_pb2 import SyNone as None_PB
from .primitive_factory import PrimitiveFactory
from .primitive_interface import PyPrimitive
from .types import SyPrimitiveRet

NoneType = type(None)


@bind_protobuf
class _SyNone(PyPrimitive):
    def __init__(self, id: Optional[UID] = None):
        self._id: UID = id if id else UID()

    @property
    def id(self) -> UID:
        """We reveal PyPrimitive.id as a property to discourage users and
        developers of Syft from modifying .id attributes after an object
        has been initialized.

        :return: returns the unique id of the object
        :rtype: UID
        """
        return self._id

    def upcast(self) -> NoneType:
        return None

    def __eq__(self, other: Any) -> SyPrimitiveRet:
        if isinstance(other, _SyNone):
            return PrimitiveFactory.generate_primitive(value=True)

        if other is None:
            return PrimitiveFactory.generate_primitive(value=True)

        res = None.__eq__(other)
        return PrimitiveFactory.generate_primitive(value=res)

    def __hash__(self) -> SyPrimitiveRet:
        res = None.__hash__()
        return PrimitiveFactory.generate_primitive(value=res)

    def _object2proto(self) -> None_PB:
        none_pb = None_PB()
        none_pb.id.CopyFrom(serialize(obj=self.id))
        return none_pb

    @staticmethod
    def _proto2object(proto: None_PB) -> "_SyNone":
        none_id: UID = deserialize(blob=proto.id)

        de_none = _SyNone()
        de_none._id = none_id

        return de_none

    @staticmethod
    def get_protobuf_schema() -> GeneratedProtocolMessageType:
        return None_PB


SyNone = _SyNone()



# stdlib
import functools
from typing import Any as TypeAny
from typing import List as TypeList
from typing import Tuple as TypeTuple

# third party
import openmined_psi

# syft relative
from ...ast import add_classes
from ...ast import add_methods
from ...ast import add_modules
from ...ast.globals import Globals
from ...generate_wrapper import GenerateProtobufWrapper
from ..util import generic_update_ast

LIB_NAME = "openmined_psi"
PACKAGE_SUPPORT = {
    "lib": LIB_NAME,
    "python": {"max_version": (3, 9, 99)},
}


def create_ast(client: TypeAny = None) -> Globals:
    ast = Globals(client=client)

    modules: TypeList[TypeTuple[str, TypeAny]] = [("openmined_psi", openmined_psi)]

    classes: TypeList[TypeTuple[str, str, TypeAny]] = [
        ("openmined_psi.client", "openmined_psi.client", openmined_psi.client),
        ("openmined_psi.server", "openmined_psi.server", openmined_psi.server),
        (
            "openmined_psi.ServerSetup",
            "openmined_psi.ServerSetup",
            openmined_psi.ServerSetup,
        ),
        (
            "openmined_psi.Request",
            "openmined_psi.Request",
            openmined_psi.Request,
        ),
        (
            "openmined_psi.Response",
            "openmined_psi.Response",
            openmined_psi.Response,
        ),
    ]

    GenerateProtobufWrapper(
        cls_pb=openmined_psi.ServerSetup,
        import_path="openmined_psi.ServerSetup",
    )
    GenerateProtobufWrapper(
        cls_pb=openmined_psi.Request,
        import_path="openmined_psi.Request",
    )
    GenerateProtobufWrapper(
        cls_pb=openmined_psi.Response,
        import_path="openmined_psi.Response",
    )

    methods = [
        ("openmined_psi.client.CreateWithNewKey", "openmined_psi.client"),
        ("openmined_psi.client.CreateRequest", "openmined_psi.Request"),
        ("openmined_psi.client.GetIntersection", "syft.lib.python.List"),
        ("openmined_psi.client.GetIntersectionSize", "syft.lib.python.Int"),
        ("openmined_psi.server.CreateWithNewKey", "openmined_psi.server"),
        (
            "openmined_psi.server.CreateSetupMessage",
            "openmined_psi.ServerSetup",
        ),
        ("openmined_psi.server.ProcessRequest", "openmined_psi.Response"),
    ]

    add_modules(ast, modules)
    add_classes(ast, classes)
    add_methods(ast, methods)

    for klass in ast.classes:
        klass.create_pointer_class()
        klass.create_send_method()
        klass.create_storable_object_attr_convenience_methods()

    return ast


update_ast = functools.partial(generic_update_ast, LIB_NAME, create_ast)



"""
The purpose of this application is to allow us to dev and test PySyft
functionality on an actual local network. This is NOT meant to be run in
production (that's the *actual* grid's job).

For example:
$ python src/syft/grid/example_nodes/network.py

"""
# stdlib
import os
import sys

# third party
import flask
from flask import Flask
from flask import Response
from nacl.encoding import HexEncoder

# syft absolute
from syft import serialize
from syft.core.common.message import SignedImmediateSyftMessageWithReply
from syft.core.common.message import SignedImmediateSyftMessageWithoutReply
from syft.core.common.serde.deserialize import _deserialize
from syft.core.node.network.network import Network
from syft.grid.services.signaling_service import PullSignalingService
from syft.grid.services.signaling_service import PushSignalingService
from syft.grid.services.signaling_service import RegisterDuetPeerService
from syft.logger import info

app = Flask(__name__)

network = Network(name="om-net")

network.immediate_services_without_reply.append(PushSignalingService)
network.immediate_services_with_reply.append(PullSignalingService)
network.immediate_services_with_reply.append(RegisterDuetPeerService)
network._register_services()  # re-register all services including SignalingService


@app.route("/metadata")
def get_metadata() -> flask.Response:
    metadata = network.get_metadata_for_client()
    metadata_proto = serialize(metadata)
    r = Response(
        response=metadata_proto.SerializeToString(),
        status=200,
    )
    r.headers["Content-Type"] = "application/octet-stream"
    return r


@app.route("/", methods=["POST"])
def process_network_msgs() -> flask.Response:
    data = flask.request.get_data()
    obj_msg = _deserialize(blob=data, from_bytes=True)
    if isinstance(obj_msg, SignedImmediateSyftMessageWithReply):
        info(
            f"Signaling server SignedImmediateSyftMessageWithReply: {obj_msg.message} watch"
        )
        reply = network.recv_immediate_msg_with_reply(msg=obj_msg)
        r = Response(response=serialize(reply, to_bytes=True), status=200)
        r.headers["Content-Type"] = "application/octet-stream"
        return r
    elif isinstance(obj_msg, SignedImmediateSyftMessageWithoutReply):
        info(
            f"Signaling server SignedImmediateSyftMessageWithoutReply: {obj_msg.message} watch"
        )
        network.recv_immediate_msg_without_reply(msg=obj_msg)
        r = Response(status=200)
        return r
    else:
        info(
            f"Signaling server SignedImmediateSyftMessageWithoutReply: {obj_msg.message} watch"
        )
        network.recv_eventual_msg_without_reply(msg=obj_msg)
        r = Response(status=200)
        return r


def run() -> None:
    global network

    IP_MODE = os.getenv("IP_MODE", "IPV4")  # default to ipv4
    if len(sys.argv) > 1:
        IP_MODE = sys.argv[1]

    IP_MODE = "IPV6" if IP_MODE == "IPV6" else "IPV4"
    # this signing_key is to aid in local development and is not used in the real
    # PyGrid implementation
    HOST = "0.0.0.0" if IP_MODE == "IPV4" else "::"  # nosec
    PORT = os.getenv("PORT", 5000)

    print("====================================")
    print("========== NODE ROOT KEY ===========")
    print("====================================")
    print(network.signing_key.encode(encoder=HexEncoder).decode("utf-8"), "\n")

    print(f"Using {IP_MODE} and listening on port {PORT}")

    app.run(host=HOST, port=int(PORT))


run()



# stdlib
from typing import Any
from typing import Union

# third party
from syft_proto.types.torch.v1.tensor_data_pb2 import TensorData as TensorData_PB
from syft_proto.types.torch.v1.tensor_pb2 import TorchTensor as TorchTensor_PB
import torch as th

# Torch dtypes to string (and back) mappers
TORCH_DTYPE_STR = {
    th.uint8: "uint8",
    th.int8: "int8",
    th.int16: "int16",
    th.int32: "int32",
    th.int64: "int64",
    th.float16: "float16",
    th.float32: "float32",
    th.float64: "float64",
    th.complex32: "complex32",
    th.complex64: "complex64",
    th.complex128: "complex128",
    th.bool: "bool",
    th.qint8: "qint8",
    th.quint8: "quint8",
    th.qint32: "qint32",
    th.bfloat16: "bfloat16",
}
TORCH_STR_DTYPE = {name: cls for cls, name in TORCH_DTYPE_STR.items()}


def set_protobuf_id(field: Any, id: Union[int, str]) -> None:
    if isinstance(id, str):
        field.id_str = id
    else:
        field.id_int = id


def get_protobuf_id(field: Any) -> Union[int, str]:
    return getattr(field, field.WhichOneof("id"))


def serialize_tensor(tensor: th.Tensor) -> TorchTensor_PB:
    """
    This method converts a Torch tensor into a serialized tensor
    using Protobuf.

    Args:
        tensor (th.Tensor): an input tensor to be serialized

    Returns:
        protobuf_obj: Protobuf version of torch tensor.
    """
    dtype = TORCH_DTYPE_STR[tensor.dtype]

    tensor_data = TensorData_PB()

    if tensor.is_quantized:
        tensor_data.is_quantized = True
        tensor_data.scale = tensor.q_scale()
        tensor_data.zero_point = tensor.q_zero_point()
        data = th.flatten(tensor).int_repr().tolist()
    else:
        data = th.flatten(tensor).tolist()

    tensor_data.dtype = dtype
    tensor_data.shape.dims.extend(tensor.size())
    getattr(tensor_data, "contents_" + dtype).extend(data)

    protobuf_tensor = TorchTensor_PB()
    set_protobuf_id(protobuf_tensor.id, getattr(tensor, "id", 1))

    protobuf_tensor.serializer = TorchTensor_PB.Serializer.SERIALIZER_ALL
    protobuf_tensor.contents_data.CopyFrom(tensor_data)
    protobuf_tensor.tags.extend(getattr(tensor, "tags", []))
    return protobuf_tensor


def deserialize_tensor(protobuf_tensor: TorchTensor_PB) -> th.Tensor:
    """
    This method converts a Protobuf torch tensor back into a
    Torch tensor.

    Args:
        protobuf_tensor (bin): Protobuf message of torch tensor.

    Returns:
        tensor (th.Tensor): a torch tensor converted from Protobuf
    """
    tensor_id = get_protobuf_id(protobuf_tensor.id)
    tags = protobuf_tensor.tags
    description = protobuf_tensor.description

    contents_type = protobuf_tensor.WhichOneof("contents")
    tensor_data_pb = getattr(protobuf_tensor, contents_type)

    size = tuple(tensor_data_pb.shape.dims)
    data = getattr(tensor_data_pb, "contents_" + tensor_data_pb.dtype)

    if tensor_data_pb.is_quantized:
        # Drop the 'q' from the beginning of the quantized dtype to get the int type
        dtype = TORCH_STR_DTYPE[tensor_data_pb.dtype[1:]]
        int_tensor = th.tensor(data, dtype=dtype).reshape(size)
        # Automatically converts int types to quantized types
        tensor = th._make_per_tensor_quantized_tensor(
            int_tensor, tensor_data_pb.scale, tensor_data_pb.zero_point
        )
    else:
        dtype = TORCH_STR_DTYPE[tensor_data_pb.dtype]
        tensor = th.tensor(data, dtype=dtype).reshape(size)

    tensor.id = tensor_id
    tensor.tags = set(tags)
    tensor.description = description

    return tensor



"""In this test suite, we evaluate the UID class. For more info
on the UID class and its purpose, please see the documentation
in the class itself.

Table of Contents:
    - INITIALIZATION: tests for various ways UID can/can't be initialized
    - CLASS METHODS: tests for the use of UID's class methods
    - SERDE: test for serialization and deserialization of UID.

"""

# stdlib
import uuid

# third party
import pytest

# syft absolute
import syft as sy
from syft.core.common import UID
from syft.core.common.serde import _deserialize
from syft.core.common.serde import _serialize
from syft.core.common.uid import uuid_type

# --------------------- INITIALIZATION ---------------------


def test_uid_creates_value_if_none_provided() -> None:
    """Tests that the UID class will create an ID if none is provided."""

    uid = UID()
    assert uid.value is not None
    assert isinstance(uid.value, uuid_type)


def test_uid_creates_value_if_try_to_init_none() -> None:
    """Tests that the UID class will create an ID if you explicitly try to init with None"""

    uid = UID(value=None)
    assert uid.value is not None
    assert isinstance(uid.value, uuid_type)


# --------------------- CLASS METHODS ---------------------


def test_uid_comparison() -> None:
    """Tests that two UIDs can be compared and will correctly evaluate"""

    uid1 = UID()
    uid2 = UID()

    # first do some basic checks
    assert uid1 == uid1
    assert uid1 != uid2

    # since the comparison should be based on the underlying value
    # let's make sure it's comparing based on the value instead of
    # just based on some other attribute of the object.
    uid2.value = uid1.value
    assert uid1 == uid2


def test_uid_hash() -> None:
    """Tests that a UID hashes correctly. If this test fails then it
    means that the uuid.UUID library changed or we tried to swap it out
    for something else. Are you sure you want to do this?"""

    uid = UID(value=uuid.UUID(int=333779996850170035686993356951732753684))
    assert hash(uid) == 1705855162796767136
    assert hash(uid.value.int) == 1705855162796767136

    fake_dict = {}
    fake_dict[uid] = "Just testing we can use it as a key in a dictionary"


def test_to_string() -> None:
    """Tests that UID generates an intuitive string."""

    uid = UID(value=uuid.UUID(int=333779996850170035686993356951732753684))

    assert str(uid) == "<UID: fb1bb0675bb74c49becee700ab0a1514>"
    assert uid.__repr__() == "<UID: fb1bb0675bb74c49becee700ab0a1514>"


def test_from_string() -> None:
    """Tests that UID can be deserialized by a human readable string."""

    uid_str = "fb1bb067-5bb7-4c49-bece-e700ab0a1514"
    uid = UID.from_string(value=uid_str)
    uid_comp = UID(value=uuid.UUID(int=333779996850170035686993356951732753684))

    assert str(uid) == "<UID: fb1bb0675bb74c49becee700ab0a1514>"
    assert uid.__repr__() == "<UID: fb1bb0675bb74c49becee700ab0a1514>"
    assert uid == uid_comp


def test_from_string_exception() -> None:
    """Tests that UID throws exception when invalid string is given. """

    with pytest.raises(Exception):
        UID.from_string(value="Hello world")


# --------------------- SERDE ---------------------


def test_uid_default_serialization() -> None:
    """Tests that default UID serialization works as expected - to Protobuf"""

    uid = UID(value=uuid.UUID(int=333779996850170035686993356951732753684))
    blob = _serialize(obj=uid)
    _ = _deserialize(blob=blob)
    assert sy.serialize(uid) == blob


def test_uid_default_deserialization() -> None:
    """Tests that default UID deserialization works as expected - from Protobuf"""

    uid = UID(value=uuid.UUID(int=333779996850170035686993356951732753684))
    blob = _serialize(obj=uid)
    obj = sy.deserialize(blob=blob)
    assert obj == UID(value=uuid.UUID(int=333779996850170035686993356951732753684))


def test_uid_proto_serialization() -> None:
    """Tests that proto UID serialization works as expected"""

    uid = UID(value=uuid.UUID(int=333779996850170035686993356951732753684))

    blob = _serialize(obj=uid)

    assert sy.serialize(uid, to_proto=True) == blob
    assert sy.serialize(uid, to_proto=True) == blob
    assert sy.serialize(uid, to_proto=True) == blob


def test_uid_proto_deserialization() -> None:
    """Tests that proto UID deserialization works as expected"""

    uid = UID(value=uuid.UUID(int=333779996850170035686993356951732753684))
    blob = _serialize(obj=uid)

    obj = sy.deserialize(blob=blob, from_proto=True)
    assert obj == UID(value=uuid.UUID(int=333779996850170035686993356951732753684))


def test_uid_binary_serialization() -> None:
    """Tests that binary UID serializes as expected"""

    uid = UID(value=uuid.UUID(int=333779996850170035686993356951732753684))
    blob = (
        b"\n\x18syft.core.common.uid.UID\x12\x12\n\x10\xfb\x1b\xb0"
        + b"g[\xb7LI\xbe\xce\xe7\x00\xab\n\x15\x14"
    )

    assert sy.serialize(uid, to_bytes=True) == blob
    assert sy.serialize(uid, to_bytes=True) == blob
    assert sy.serialize(uid, to_bytes=True) == blob


def test_uid_binary_deserialization() -> None:
    """Test that binary deserialization works as expected"""

    blob = (
        b"\n\x18syft.core.common.uid.UID\x12\x12\n\x10\xfb\x1b\xb0"
        + b"g[\xb7LI\xbe\xce\xe7\x00\xab\n\x15\x14"
    )

    obj = sy.deserialize(blob=blob, from_bytes=True)
    assert obj == UID(value=uuid.UUID(int=333779996850170035686993356951732753684))



# stdlib
import gc

# third party
import pytest
import torch as th

# syft absolute
import syft as sy
from syft.core.node.common.service.auth import AuthorizationException


@pytest.mark.slow
def test_torch_remote_tensor_register(
    node: sy.VirtualMachine, client: sy.VirtualMachineClient
) -> None:
    """ Test if sending a tensor will be registered on the remote worker. """
    x = th.tensor([-1, 0, 1, 2, 3, 4])
    ptr = x.send(client, pointable=False)

    assert len(node.store) == 1

    ptr = x.send(client, pointable=False)
    gc.collect()

    # the previous objects get deleted because we overwrite
    # ptr - we send a message to delete that object
    assert len(node.store) == 1

    ptr.get()
    assert len(node.store) == 0  # Get removes the object


def test_torch_remote_tensor_with_send(
    node: sy.VirtualMachine, client: sy.VirtualMachineClient
) -> None:
    """Test sending tensor on the remote worker with send method."""

    x = th.tensor([-1, 0, 1, 2, 3, 4])
    ptr = x.send(client)

    assert len(node.store) == 1

    data = ptr.get()

    assert len(node.store) == 0  # Get removes the object

    assert x.equal(data)  # Check if send data and received data are equal


def test_torch_serde() -> None:

    x = th.tensor([1.0, 2, 3, 4], requires_grad=True)

    # This is not working currently:
    # (x * 2).sum().backward()
    # But pretend we have .grad
    x.grad = th.randn_like(x)

    blob = sy.serialize(x)

    x2 = sy.deserialize(blob=blob)

    assert (x == x2).all()


@pytest.mark.slow
def test_torch_no_read_permissions(
    client: sy.VirtualMachineClient, root_client: sy.VirtualMachineClient
) -> None:

    x = th.tensor([1, 2, 3, 4])

    # root user of Bob's machine sends a tensor
    ptr = x.send(root_client)

    # guest creates a pointer to that object (assuming the client can guess/infer the ID)
    ptr.client = client

    # this should trigger an exception
    with pytest.raises(AuthorizationException):
        ptr.get()

    x = th.tensor([1, 2, 3, 4])

    # root user of Bob's machine sends a tensor
    ptr = x.send(root_client)

    # but if root bob asks for it it should be fine
    x2 = ptr.get()

    assert (x == x2).all()

    assert x.grad == x2.grad


def test_torch_garbage_collect(
    node: sy.VirtualMachine, client: sy.VirtualMachineClient
) -> None:
    """
    Test if sending a tensor and then deleting the pointer removes the object
    from the remote worker.
    """
    x = th.tensor([-1, 0, 1, 2, 3, 4])
    ptr = x.send(client, pointable=False)

    assert len(node.store) == 1

    # "del" only decrements the counter and the garbage collector plays the role of the reaper
    del ptr

    # Make sure __del__ from Pointer is called
    gc.collect()

    assert len(node.store) == 0



# stdlib
from argparse import ArgumentParser
import os


def makedirs(name):
    """helper function for python 2 and 3 to call os.makedirs()
    avoiding an error if the directory to be created already exists"""

    # stdlib
    import errno
    import os

    try:
        os.makedirs(name)
    except OSError as ex:
        if ex.errno == errno.EEXIST and os.path.isdir(name):
            # ignore existing directory
            pass
        else:
            # a different error occured
            raise


def get_args():
    parser = ArgumentParser(description="PyTorch/torchtext SNLI example")
    parser.add_argument(
        "--epochs",
        type=int,
        default=50,
        help="the number of total epochs to run; default: 50"
    )
    parser.add_argument(
        "--batch_size",
        type=int,
        default=128,
        help="batch size; default: 128"
    )
    parser.add_argument(
        "--d_embed",
        type=int,
        default=100,
        help="the size of each embedding vector; default: 100"
    )
    parser.add_argument(
        "--d_proj",
        type=int,
        default=300,
        help="the size of each projection layer; default: 300"
    )
    parser.add_argument(
        "--d_hidden",
        type=int,
        default=300,
        help="the number of features in the hidden state; default: 300",
    )
    parser.add_argument(
        "--n_layers",
        type=int,
        default=1,
        help="the number of recurrent layers; default: 50",
    )
    parser.add_argument(
        "--log_every",
        type=int,
        default=50,
        help="iteration period to output log; default: 50"
    )
    parser.add_argument(
        "--lr",
        type=float,
        default=0.001,
        help="initial learning rate; default: 0.001"
    )
    parser.add_argument(
        "--dev_every",
        type=int,
        default=1000,
        help="log period of validation results; default: 1000"
    )
    parser.add_argument(
        "--save_every",
        type=int,
        default=1000,
        help="model checkpoint period; default: 1000"
    )
    parser.add_argument(
        "--dp_ratio",
        type=int,
        default=0.2,
        help="probability of an element to be zeroed; default: 0.2",
    )
    parser.add_argument(
        "--no-bidirectional",
        action="store_false",
        dest="birnn",
        help="disable bidirectional LSTM.",
    )
    parser.add_argument(
        "--preserve-case",
        action="store_false",
        dest="lower",
        help="case-sensitivity."
    )
    parser.add_argument(
        "--no-projection",
        action="store_false",
        dest="projection",
        help="disable projection layer.",
    )
    parser.add_argument(
        "--train_embed",
        action="store_false",
        dest="fix_emb",
        help="enable embedding word training.",
    )
    parser.add_argument(
        "--gpu",
        type=int,
        default=0,
        help="gpu id to use; default: 0"
    )
    parser.add_argument(
        "--save_path",
        type=str,
        default="results",
        help="save path of results; default: \"results\""
    )
    parser.add_argument(
        "--vector_cache",
        type=str,
        default=os.path.join(os.getcwd(), ".vector_cache/input_vectors.pt"),
        help='name of vector cache directory, which saved input word-vectors. \
            default=os.path.join(os.getcwd(), \".vector_cache/input_vectors.pt\")'
    )
    parser.add_argument(
        "--word_vectors",
        type=str,
        default="glove.6B.100d",
        help="one of or a list containing instantiations of the GloVe, CharNGram, or Vectors classes.\
            Alternatively, one of or a list of available pretrained vectors: \
            charngram.100d fasttext.en.300d fasttext.simple.300d \
            glove.42B.300d glove.840B.300d glove.twitter.27B.25d \
            glove.twitter.27B.50d glove.twitter.27B.100d glove.twitter.27B.200d \
            glove.6B.50d glove.6B.100d glove.6B.200d glove.6B.300d \
            default: glove.6B.100d"
    )
    parser.add_argument(
        "--resume_snapshot",
        type=str,
        default="",
        help="model snapshot to resume; default:\"\""
    )
    parser.add_argument(
        "--dry-run",
        action="store_true",
        default=False,
        help="run only a few iterations; default: False"
    )
    args = parser.parse_args()
    return args



# stdlib
from typing import Any
from typing import List
from typing import Union

# third party
import torch as th

# syft absolute
import syft as sy


def test_module() -> None:
    torch = sy.lib.torch.torch
    nn = torch.nn
    F = torch.nn.functional

    # manually constructed Module with PyTorch external API
    class Net:
        modules: List[Any] = []
        training = False

        def __init__(self) -> None:
            super(Net, self).__init__()
            self.conv1 = nn.Conv2d(1, 32, 3, 1)
            self.conv2 = nn.Conv2d(32, 64, 3, 1)
            self.dropout1 = nn.Dropout2d(0.25)
            self.dropout2 = nn.Dropout2d(0.5)
            self.fc1 = nn.Linear(9216, 128)
            self.fc2 = nn.Linear(128, 10)

            # add to modules list
            self.modules.append(self.conv1)
            self.modules.append(self.conv2)
            self.modules.append(self.dropout1)
            self.modules.append(self.dropout2)
            self.modules.append(self.fc1)
            self.modules.append(self.fc2)

        def train(self, mode: bool = True) -> Any:
            self.training = mode
            for module in self.modules:
                module.train(mode)
            return self

        def eval(self) -> Any:
            return self.train(False)

        def forward(self, x: Any) -> Any:
            x = self.conv1(x)
            x = F.relu(x)
            x = self.conv2(x)
            x = F.relu(x)
            x = F.max_pool2d(x, 2)
            x = self.dropout1(x)
            x = torch.flatten(x, 1)
            x = self.fc1(x)
            x = F.relu(x)
            x = self.dropout2(x)
            x = self.fc2(x)
            output = F.log_softmax(x, dim=1)
            return output

        def __call__(self, input: Any) -> Any:
            return self.forward(input)

        # local list of remote ListPointers of TensorPointers
        def parameters(self, recurse: bool = True) -> List:
            params_list = torch.python.List()
            for module in self.modules:
                param_pointers = module.parameters()
                params_list += param_pointers

            return params_list

        def cuda(self, device: Any) -> "Net":
            for module in self.modules:
                module.cuda(device)
            return self

        def cpu(self) -> "Net":
            for module in self.modules:
                module.cpu()
            return self

    # sy.Module version with equivalent external interface
    class SyNet(sy.Module):
        def __init__(self, torch_ref: Any) -> None:
            super(SyNet, self).__init__(torch_ref=torch_ref)
            self.conv1 = torch_ref.nn.Conv2d(1, 32, 3, 1)
            self.conv2 = torch_ref.nn.Conv2d(32, 64, 3, 1)
            self.dropout1 = torch_ref.nn.Dropout2d(0.25)
            self.dropout2 = torch_ref.nn.Dropout2d(0.5)
            self.fc1 = torch_ref.nn.Linear(9216, 128)
            self.fc2 = torch_ref.nn.Linear(128, 10)

        def forward(self, x: Any) -> Any:
            x = self.conv1(x)
            x = self.torch_ref.nn.functional.relu(x)
            x = self.conv2(x)
            x = self.torch_ref.nn.functional.relu(x)
            x = self.torch_ref.nn.functional.max_pool2d(x, 2)
            x = self.dropout1(x)
            x = self.torch_ref.flatten(x, 1)
            x = self.fc1(x)
            x = self.torch_ref.nn.functional.relu(x)
            x = self.dropout2(x)
            x = self.fc2(x)
            output = self.torch_ref.nn.functional.log_softmax(x, dim=1)
            return output

    vanilla_model = Net()
    sy_model = SyNet(torch_ref=th)
    models: List[Union[Net, SyNet]] = [sy_model, vanilla_model]
    for model in models:
        assert hasattr(model, "__call__")
        assert hasattr(model, "__init__")
        assert hasattr(model, "cpu")
        assert hasattr(model, "cuda")
        assert hasattr(model, "eval")
        assert hasattr(model, "forward")
        assert hasattr(model, "parameters")
        assert hasattr(model, "train")

        # check training is off
        assert model.training is False

        model.train()

        assert model.training is True

        model.eval()  # type: ignore

        assert model.training is False

    # assert vanilla_model.modules == sy_model._modules
    assert len(vanilla_model.modules) == len(sy_model.modules)
    assert len(sy_model.modules) == 6
    for syft_module, vanilla_module in zip(
        sy_model.modules.values(), vanilla_model.modules
    ):
        # we cant do module == module, but the str repr of the modules should be equal
        assert str(syft_module) == str(vanilla_module)
